      SUBROUTINE DLWQWQ ( NOTOT  , NOSYS  , NOSEG  , NOPA   , NOSFUN ,
     *                    VOLUME , CONC   , CONS   , PARAM  , FUNC   ,
c  Dynamic mode!!!
c     *                    SEGFUN , DERIV  , ITIME  , IDT    , ASMASS ,
c  Steady mode!!!
c     *                    SEGFUN , DERIV  , ISYS   , IDT    , ASMASS ,
     *                    SEGFUN , DERIV  , ITIME  , IDT    , ASMASS ,
     *                    IBFLAG , SYNAME , NOCONS , NOFUN  , CONAME ,
     *                    PANAME , FUNAME , SFNAME , NODUMP , IDUMP  )
C
C     Deltares     SECTOR WATERRESOURCES AND ENVIRONMENT
C
C     CREATED:    march 1988 by L.Postma
c     MODIFIED:   january 2011 by J. van Gils

C Voor een stof met naam xxx in DELWAQ:

C - de default is "geen afbraak
C - als er een constante is met de naam "k_xxx" haal ik de afbraakconstante uit die constante
C - als er een parameter is met de naam "k_xxx" haal ik de afbraakconstante uit die parameter.
C - parameter gaat voor constante
C Verder is er nog een constante "Processes_Dynamic" (waarde irrelevant) die ervoor zorgt dat 
C de dynamische variant wordt gebruikt in plaats van de steady state variant
C
C     FUNCTION            : WATERQUALITY subroutine
C
C
C     PARAMETERS          :
C
C     NAME    KIND     LENGTH     FUNCT.  DESCRIPTION
C     ----    -----    ------     ------- -----------
C     NOTOT   INTEGER       1     INPUT   Total number of substances
C     NOSYS   INTEGER       1     INPUT   Number of active substances
C     NOSEG   INTEGER       1     INPUT   Nr. of computational elements
C     NOPA    INTEGER       1     INPUT   Number of parameters
C     NOSFUN  INTEGER       1     INPUT   Number of segment functions
C     VOLUME  REAL      NOSEG     INPUT   Segment volumes
C     CONC    REAL   NOTOT*NOSEG  INPUT   Model concentrations, first
C                                         NOSYS are active and expressed
C                                         as concentrations, others are
C                                         inactive, expressed in MASS!!!
C     CONS    REAL          *     IN/OUT  Model constants
C     PARAM   REAL    NOPA*NOSEG  IN/OUT  Model parameters
C     FUNC    REAL          *     IN/OUT  Model functions at ITIME
C     SEGFUN  REAL   NOSEG*NOSFUN IN/OUT  Segment functions at ITIME
C     DERIV   REAL   NOTOT*NOSEG  OUTPUT  Model derivatives
C     ITIME   INTEGER       1     INPUT   Time in system clock units
C     IDT     INTEGER       1     INPUT   Time step system clock units
C     ASMASS  REAL  NOTOT*NOSEG*? IN/OUT  mass balance per comp. elems.
C                                         if IBFLAG = 1
C     IBFLAG  INTEGER    1        INPUT   if 1 then mass balance p.c.e.
C     SYNAME  CHAR*20    NOTOT    INPUT   names of systems
C     NOCONS  INTEGER       1     INPUT   Number of constants used
C     NOFUN   INTEGER       1     INPUT   Number of functions ( user )
C     CONAME  CHAR*20   NOCONS    INPUT   Constant names
C     PANAME  CHAR*20   NOPA      INPUT   Parameter names
C     FUNAME  CHAR*20   NOFUN     INPUT   Function names
C     SFNAME  CHAR*20   NOSFUN    INPUT   Segment function names
C     NODUMP  INTEGER       1     INPUT   Number of monitor points
C     IDUMP   INTEGER   NODUMP    INPUT   Segment numbers monitor points
C
C     ==================================================================
C
C          DIMENSION  PARAM (NOPA,NOSEG) , SEGFUN(NOSEG,NOSFUN)
C
C     NOTE that previous dimension statement is only correct if
C          PARAM and NOSFUN are larger than zero, which is not
C          always the case. The user may insert 2-D dimensioning
C          for applications with non-zero PARAM and/or NOSFUN.
C
      CHARACTER*20 SYNAME (NOTOT), CONAME (*), PANAME (*),
     &             FUNAME (*)    , SFNAME (*)
      DIMENSION    VOLUME(NOSEG) , CONC  (NOTOT,NOSEG) , CONS(*) ,
     &             FUNC  (  *  ) , DERIV (NOTOT,NOSEG) ,
     &             PARAM (NOPA,NOSEG) , SEGFUN(NOSEG,NOSFUN),
     &             ASMASS(NOTOT,NOSEG,*), IDUMP(*)
C
C          user-defined waterquality processes
C                -  expressed in mass per time
C                -  positive derivative is growth
C                -  DERIV is zero at entrance
C                -  CONC   may be changed but it has no effect
C                -  VOLUME may NEVER be changed
c          steady state: DERIVE holds zero order term
c                        CONC first order term
C                        repeated call for individual substances (ISYS)

C     Local declarations
C
      LOGICAL, save ::      FIRST = .true.
      LOGICAL, save ::      DYNAMIC
      REAL, allocatable, save ::         RK(:,:)
      REAL, allocatable, save ::         KD(:,:)
      REAL, allocatable, save ::         RKT(:,:)
      REAL, allocatable, save ::         TEMP(:)
      REAL, allocatable, save ::         SS(:)
      REAL, allocatable, save ::         SURF(:)
      REAL, allocatable, save ::         VSED(:)
      INTEGER      ICDYN, ISTOF, ICDEC, IPDEC
      CHARACTER*20 DECNAM
      REAL         RKDEC, RKSET, SSLOC, VSEDLOC, KDLOC
C
      IF (FIRST) THEN
         allocate (rk(nosys,noseg))
         allocate (rkT(nosys,noseg))
         allocate (kd(nosys,noseg))
         allocate (TEMP(noseg))
         allocate (SS(noseg))
         allocate (VSED(noseg))
         allocate (SURF(noseg))
c       Check for dynamic simulation
         CALL ZOEK ( 'Processes_Dynamic   ', NOCONS, CONAME,20 ,ICDYN )
         IF ( ICDYN .LE. 0 ) THEN
            DYNAMIC = .FALSE.
            WRITE ( * , * ) ' STEADY STATE mode'
         ELSE
             DYNAMIC = .TRUE.
            WRITE ( * , * ) ' DYNAMIC mode'
         ENDIF
         
c        Initialise INPUT ITEMS
         DO ISEG = 1, NOSEG
            TEMP = 20.
            SURF = 1.0
            SS = 0.
            VSED = 0.0
            DO ISTOF = 1,NOSYS
                RK(ISTOF, ISEG) = 0.0
                KD(ISTOF, ISEG) = 0.0
                RKT(ISTOF, ISEG) = 1.0
            ENDDO
         ENDDO

C ------ READ TEMP         
         DECNAM = 'Temp'
c        Check constant 
         CALL ZOEK ( DECNAM, NOCONS, CONAME,20 ,ICDEC )
         IF (ICDEC.GT.0) THEN
            DO ISEG = 1, NOSEG
               TEMP(ISEG) = CONS(ICDEC)
            ENDDO
         ENDIF
c        Check space dependent 
         CALL ZOEK ( DECNAM, NOPA, PANAME,20 ,IPDEC )
         IF (IPDEC.GT.0) THEN
            DO ISEG = 1, NOSEG
               TEMP(ISEG) = PARAM(IPDEC,ISEG)
            ENDDO
         ENDIF

C ------ READ SURF
         DECNAM = 'Surf'
c        Check constant 
         CALL ZOEK ( DECNAM, NOCONS, CONAME,20 ,ICDEC )
         IF (ICDEC.GT.0) THEN
            DO ISEG = 1, NOSEG
               SURF(ISEG) = CONS(ICDEC)
            ENDDO
         ENDIF
c        Check space dependent 
         CALL ZOEK ( DECNAM, NOPA, PANAME,20 ,IPDEC )
         IF (IPDEC.GT.0) THEN
            DO ISEG = 1, NOSEG
               SURF(ISEG) = PARAM(IPDEC,ISEG)
            ENDDO
         ENDIF
         
C ------ READ SS
         DECNAM = 'SS'
c        Check constant 
         CALL ZOEK ( DECNAM, NOCONS, CONAME,20 ,ICDEC )
         IF (ICDEC.GT.0) THEN
            DO ISEG = 1, NOSEG
               SS(ISEG) = CONS(ICDEC)
            ENDDO
         ENDIF
c        Check space dependent 
         CALL ZOEK ( DECNAM, NOPA, PANAME,20 ,IPDEC )
         IF (IPDEC.GT.0) THEN
            DO ISEG = 1, NOSEG
               SS(ISEG) = PARAM(IPDEC,ISEG)
            ENDDO
         ENDIF
         
C ------ READ VSED
         DECNAM = 'VSed'
c        Check constant 
         CALL ZOEK ( DECNAM, NOCONS, CONAME,20 ,ICDEC )
         IF (ICDEC.GT.0) THEN
            DO ISEG = 1, NOSEG
               VSED(ISEG) = CONS(ICDEC)
            ENDDO
         ENDIF
c        Check space dependent 
         CALL ZOEK ( DECNAM, NOPA, PANAME,20 ,IPDEC )
         IF (IPDEC.GT.0) THEN
            DO ISEG = 1, NOSEG
               VSED(ISEG) = PARAM(IPDEC,ISEG)
            ENDDO
         ENDIF

c        Loop over substances
         DO ISTOF = 1,NOSYS
         
C --------- READ DECAY RATE
            DECNAM = ' '
            DECNAM = 'k_'//SYNAME(ISTOF)

c           Check constant decay rate
            CALL ZOEK ( DECNAM, NOCONS, CONAME,20 ,ICDEC )
            IF (ICDEC.GT.0) THEN
               DO ISEG = 1, NOSEG
                  RK(ISTOF, ISEG) = CONS(ICDEC)
               ENDDO
            ENDIF

c           Check space dependent decay rate
            CALL ZOEK ( DECNAM, NOPA, PANAME,20 ,IPDEC )
            IF (IPDEC.GT.0) THEN
               DO ISEG = 1, NOSEG
                  RK(ISTOF, ISEG) = PARAM(IPDEC,ISEG)
               ENDDO
            ENDIF

C --------- READ TEMP COEFF
            DECNAM = ' '
            DECNAM = 'kT_'//SYNAME(ISTOF)

c           Check constant 
            CALL ZOEK ( DECNAM, NOCONS, CONAME,20 ,ICDEC )
            IF (ICDEC.GT.0) THEN
               DO ISEG = 1, NOSEG
                  RKT(ISTOF, ISEG) = CONS(ICDEC)
               ENDDO
            ENDIF

c           Check space dependent 
            CALL ZOEK ( DECNAM, NOPA, PANAME,20 ,IPDEC )
            IF (IPDEC.GT.0) THEN
               DO ISEG = 1, NOSEG
                  RKT(ISTOF, ISEG) = PARAM(IPDEC,ISEG)
               ENDDO
            ENDIF
         
C --------- READ KD
            DECNAM = ' '
            DECNAM = 'kd_'//SYNAME(ISTOF)

c           Check constant 
            CALL ZOEK ( DECNAM, NOCONS, CONAME,20 ,ICDEC )
            IF (ICDEC.GT.0) THEN
               DO ISEG = 1, NOSEG
                  KD(ISTOF, ISEG) = CONS(ICDEC)
               ENDDO
            ENDIF
c           Check space dependent 
            CALL ZOEK ( DECNAM, NOPA, PANAME,20 ,IPDEC )
            IF (IPDEC.GT.0) THEN
               DO ISEG = 1, NOSEG
                  KD(ISTOF, ISEG) = PARAM(IPDEC,ISEG)
               ENDDO
            ENDIF
         ENDDO
         
C ------ SET VALUES OF TEMPERATURE DEPENDENT DECAY RATE

         DO ISTOF = 1,NOSYS
         DO ISEG = 1, NOSEG
            RKDEC = RK(ISTOF, ISEG) 
     J            * ( RKT(ISTOF, ISEG)**(TEMP(ISEG)-20.) )
            SSLOC = SS(ISEG)
            KDLOC = KD(ISTOF,ISEG)
            VSEDLOC=VSED(ISEG)
            RKSET = VSEDLOC/(VOLUME(ISEG)/SURF(ISEG))
     J             * SSLOC*KDLOC/(1.+SSLOC*KDLOC)
            RK(ISTOF, ISEG) = RKDEC + RKSET
         ENDDO
         ENDDO
 
         first = .false.
      ENDIF

      IF (DYNAMIC) THEN
     
C---------------------------------------------------------------------
C       DYNAMIC SECTION
C---------------------------------------------------------------------
        DO ISEG = 1, NOSEG
        DO ISYS = 1,NOSYS
            DERIV(ISYS, ISEG) = -RK(ISYS, ISEG) * VOLUME(ISEG) 
     j                        * conc(isys,iseg)
        ENDDO
        ENDDO
C---------------------------------------------------------------------
C       END OF DYNAMIC SECTION
C---------------------------------------------------------------------
      ELSE      
C---------------------------------------------------------------------
C       STEADY STATE SECTION
C---------------------------------------------------------------------
C
C
      ISYS = ITIME
      DO ISEG = 1, NOSEG

C --- Zero order term
        DERIV (ISYS, ISEG) = 0.0

C ---   First order term
        CONC (ISYS, ISEG ) = - RK(ISYS, ISEG) * VOLUME (ISEG)
      ENDDO
C---------------------------------------------------------------------
C       END OF STEADY STATE SECTION
C---------------------------------------------------------------------
      ENDIF      
C
      RETURN
      END
