      subroutine make_flx (casedata,lu_mes)

	use scremo_data

	type(t_casedata) :: casedata
	integer          :: lu_mes

      real, parameter :: z0 =0.1
	integer         :: n,m,k,ioqn,ioqm,ioqk,n1,n2
	real            :: localdepthtop, localdepthright
      real            :: constant, local_z
      real            :: angle, rel_local_z
	real            :: sumflow, wind_x, wind_y
	real            :: sum_x, sum_y
	real            :: du, dv
	real            :: Y_factor
	real, parameter :: pi = 3.141516

      u2d = 0.0 
      v2d = 0.0 
	if (casedata%cas.eq.1) then

	! Code for CASE 1

      n1 = 1
	n2 = nmax-1
	do m = 2,mmax-1
	do n = 1,nmax-1

      ! Background current in Y direction, dependent on depth DIRECTION REVERSED FOR CONSISTENCY WITH CASE 2

          localdepthtop = (depth(n,m-1)+depth(n,m))/2.
          v2d(n,m) = -((localdepthtop/casedata%H)**0.667)*casedata%CS     ! minus sign added
	enddo
	enddo

	! END Code for CASE 1

      elseif (casedata%cas.eq.2) then

	! Code for CASE 2

      n1 = 2
	n2 = nmax-2
	do m = 2,mmax-1
	do n = 2,nmax-1
	! set currents in X-direction
	    Y_factor = (real(n)-1.5)/real(nmax-2)
	    Y_factor = 1.-2.*Y_factor
	    localdepthright = (depth(n-1,m)+depth(n,m))/2.
	    u2d(n,m) = ((localdepthright/casedata%H)**0.667)*casedata%CS
     j                *Y_factor
	enddo
	enddo
	do m = 2,mmax-1
	do n = 2,nmax-2
	! set currents in Y-direction from water balance
	    sumflow = 0.0
	    sumflow = sumflow + u2d(n  ,m-1)*ax(n  ,m-1)*real(casedata%KG)
	    sumflow = sumflow - u2d(n  ,m  )*ax(n  ,m  )*real(casedata%KG)
	    sumflow = sumflow + v2d(n-1,m  )*ay(n-1,m  )*real(casedata%KG)
          v2d(n,m) = sumflow/ay(n,m)/real(casedata%KG)
	enddo
	enddo

	! END Code for CASE 2

	else
          write(*,*) 'Make_flx: Geometry case not implemented'
          stop (4)
	endif

	! calculate wind components relative to grid

	angle = (casedata%WD-casedata%GD)/360.*2.*pi
	wind_x = -sin(angle)*casedata%WS
	wind_y = -cos(angle)*casedata%WS
	write (lu_mes,*) 'Rel.wind angle   = ',angle/2./pi*360.
	write (lu_mes,*) 'X-component wind = ',wind_x
	write (lu_mes,*) 'Y-component wind = ',wind_y

	uc = 0.
	vc = 0.
	wc = 0.

!     Initialise 3D flow field for wind driven currents
      
	do m = 2,mmax-1
	do n = 1,nmax-1

	    ! wind driven current

	    sum_x = 0.0
	    sum_y = 0.0
          do k = 1,nlay
	        rel_local_z = (real(nlay-k)+0.5)/real(nlay)

	        ! x direction flux (right side)

              if(n.gt.1) then
	        uc(n,m,k) = wind_x*0.03*(3.*rel_local_z-2.)*rel_local_z
	        sum_x = sum_x + uc(n,m,k)
	        endif

	        ! y direction flux (top side)

              if (n.ge.n1.and.n.le.n2) then
              vc(n,m,k) = wind_y*0.03*(3.*rel_local_z-2.)*rel_local_z
	        sum_y = sum_y + vc(n,m,k)
	        endif
	    enddo

	    ! correction to enforce vertical integral = 0

          do k = 1,nlay
	        uc(n,m,k) = uc(n,m,k) - sum_x/real(nlay)
	        vc(n,m,k) = vc(n,m,k) - sum_y/real(nlay)
	    enddo

	enddo
	enddo

!     Add circulation pattern

	do m = 2,mmax-1
	do n = 1,nmax-1

          if (n.gt.1) then
	    localdepthright = (depth(n-1,m)+depth(n,m))/2.
	    else
	    localdepthright = depth(n,m)
	    endif
	    localdepthtop = (depth(n,m-1)+depth(n,m))/2.
	    sum_x = 0.0
	    sum_y = 0.0

          do k = 1,nlay
	        rel_local_z = (real(nlay-k)+0.5)/real(nlay)
              local_z = rel_local_z*localdepthright
	        constant=1./((1.+z0/localdepthright)
     j                *log(localdepthright/z0+1.)-1.)
	        du = constant*u2d(n,m)*log(local_z/z0+1.)
              uc(n,m,k) = uc(n,m,k) + du
	        sum_x = sum_x + du

              local_z = rel_local_z*localdepthtop
	        constant=1./((1.+z0/localdepthtop)
     j                *log(localdepthtop/z0+1.)-1.)
              dv = constant*v2d(n,m)*log(local_z/z0+1.)
              vc(n,m,k) = vc(n,m,k) + dv
	        sum_y = sum_y + dv
	    enddo

	    ! correction to enforce vertical integral = 2D value

          do k = 1,nlay
	        uc(n,m,k) = uc(n,m,k) - (sum_x/real(nlay)-u2d(n,m))
	        vc(n,m,k) = vc(n,m,k) - (sum_y/real(nlay)-v2d(n,m))
	    enddo
	enddo
	enddo

      ! vertical currents by closing water balance, from bottom to top

      do k = nlay,2,-1
	do m = 2,mmax-1
	do n = 2,nmax-1
	    sumflow = 0.0

	    sumflow = sumflow + uc(n  ,m-1,k)*ax(n  ,m-1)
	    sumflow = sumflow - uc(n  ,m  ,k)*ax(n  ,m  )
	    sumflow = sumflow + vc(n-1,m  ,k)*ay(n-1,m  )
	    sumflow = sumflow - vc(n  ,m  ,k)*ay(n  ,m  )
	    if (k.lt.nlay) sumflow = sumflow 
     j                   - wc(n,m,k)*deltax(n,m)*deltay(n,m)

	    wc(n,m,k-1) = -sumflow/deltax(n,m)/deltay(n,m)
	enddo
	enddo
	enddo

      ! Project on exchanges

	do k = 1,nlay
	do m = 1,mmax
	do n = 1,nmax

          call get_ioq(n,m,k,ioqn,ioqm,ioqk)

          if (ioqn.gt.0) flow(ioqn) = vc(n,m,k)*area(ioqn)
          if (ioqm.gt.0) flow(ioqm) = uc(n,m,k)*area(ioqm)
          if (ioqk.gt.0) flow(ioqk) = wc(n,m,k)*area(ioqk)

	enddo
	enddo
	enddo

      
	return
	end
