      ! program NEPFLOW

      ! global

      use scremo_data

      ! local

      character*255             :: infile            ! input file with case data
      character*255             :: hydfile           ! output HYD file for WAQ
      character*255             :: hydname           ! output HYD name for WAQ (= name HYD file)
      integer, parameter        :: lu_inp = 21
      integer, parameter        :: lu_mes = 22
      integer, parameter        :: lu_hyd = 23
	type(t_casedata)          :: casedata 
	integer                   :: itime(2), ntime, iseg, ioq
	logical, parameter        :: debug = .true.
	integer                   :: n,m,k

      ! collect input and output file names, open some files

      CALL GETARG (1, infile)
      CALL GETARG (2, hydfile)

	open (lu_inp,file=infile,status='old',err=900)
	open (lu_hyd,file=hydfile)
	open (lu_mes,file='nepflow.lst')
	hydname = hydfile(1:index(hydfile,'.hyd')-1)

	write (lu_mes,*) 'Input file: ', trim(infile)
	write (lu_mes,*) 'HYD   file: ', trim(hydfile)
	write (lu_mes,*) 'HYD   name: ', trim(hydname)

	! read input

	call getinput(lu_inp,lu_mes,casedata)
      nmax = casedata%NG+2
      mmax = casedata%MG+2
	nlay = casedata%KG
	noseg = casedata%NG*casedata%MG*casedata%KG
	nosegl = casedata%NG*casedata%MG
	noq1  = (nmax-1)*(mmax-2)* nlay
	noq2  = (nmax-2)*(mmax-1)* nlay
	noq3  = (nmax-2)*(mmax-2)*(nlay-1)
	noq   = noq1 + noq2 + noq3
	write (lu_mes,*) 'NOSEG =   : ', noseg
	write (lu_mes,*) 'NOQ =     : ', noq

      ! write skeleton hyd file

	call writehyd(lu_hyd,hydname,lu_mes)

      ! Grid & geometry

      ALLOCATE (LGRID(NMAX,MMAX))
      ALLOCATE (XB(NMAX,MMAX))
      ALLOCATE (YB(NMAX,MMAX))
      ALLOCATE (XBt(NMAX,MMAX))
      ALLOCATE (YBt(NMAX,MMAX))
	call make_lgacco (hydname,casedata)
	write (lu_mes,*) 'NBOUNL =     : ', nbounl
	nboun = -nbounl*casedata%KG
	write (lu_mes,*) 'NBOUN  =     : ', nboun
	if (debug) then
	    write (lu_mes,'(/''LGRID''/)')
	    do n=nmax,1,-1
	       write (lu_mes,'(100i5)') (lgrid(n,m),m=1,mmax)
	    enddo
	endif

      ! DELWAQ data

      allocate(ipoint(4,noq))
      allocate(length(2,noq))
      allocate(volume(noseg))
      allocate(surf(noseg))
      allocate(area(noq))
      allocate(flow(noq))

	! call geometry

	allocate (lgrid3D(nmax,mmax,nlay))
	allocate (deltax(nmax,mmax))
	allocate (deltay(nmax,mmax))
	allocate (deltaz(nmax,mmax))
	allocate (depth(nmax,mmax))
	allocate (ax(nmax,mmax))
	allocate (ay(nmax,mmax))
	ipoint = 0
	call make_geo (casedata)
	if (debug) then
	    write (lu_mes,'(/''DELTAX''/)')
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.0)') (deltax(n,m),m=1,mmax)
	    enddo
	    write (lu_mes,'(/''DELTAY''/)')
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.0)') (deltay(n,m),m=1,mmax)
	    enddo
	    write (lu_mes,'(/''DELTAZ''/)')
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.2)') (deltaz(n,m),m=1,mmax)
	    enddo
	    write (lu_mes,'(/''DEPTH''/)')
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.2)') (depth(n,m),m=1,mmax)
	    enddo
	    write (lu_mes,'(/''AX''/)')
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.0)') (ax(n,m),m=1,mmax)
	    enddo
	    write (lu_mes,'(/''AY''/)')
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.0)') (ay(n,m),m=1,mmax)
	    enddo
	endif

	! call fluxes

      flow = 0.0
	allocate (uc(nmax,mmax,nlay))
	allocate (vc(nmax,mmax,nlay))
	allocate (wc(nmax,mmax,nlay))
	allocate (u2d(nmax,mmax))
	allocate (v2d(nmax,mmax))
	call make_flx(casedata,lu_mes)
	if (debug) then
	    write (lu_mes,'(/''U2d''/)')
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.4)') (u2d(n,m),m=1,mmax)
	    enddo
	    write (lu_mes,'(/''V2d''/)')
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.4)') (v2d(n,m),m=1,mmax)
	    enddo
	    do k=1,nlay
	    write (lu_mes,'(/''U in layer '',i2/)') k
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.4)') (uc(n,m,k),m=1,mmax)
	    enddo
	    enddo
	    do k=1,nlay
	    write (lu_mes,'(/''V in layer '',i2/)') k
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.4)') (vc(n,m,k),m=1,mmax)
	    enddo
	    enddo
	    do k=1,nlay
	    write (lu_mes,'(/''W in layer '',i2/)') k
	    do n=nmax,1,-1
	       write (lu_mes,'(100f10.4)') (wc(n,m,k),m=1,mmax)
	    enddo
	    enddo
	endif

      if (debug) then
	write (lu_mes,'(/''Segment      Volume        Surf'')')
	write (lu_mes,'(i7,2f12.0)')
     j (iseg,volume(iseg),surf(iseg),iseg=1,noseg)
	write (lu_mes,'(/
     j ''  Uitw  From    To  Len1  Len2      Area      Flow'')')
	write (lu_mes,'(3i6,2f6.0,f10.0,f10.2)')
     j (ioq,ipoint(1,ioq),ipoint(2,ioq),length(1,ioq),length(2,ioq),
     j    area(ioq),flow(ioq),ioq=1,noq)
      endif
      
c     discharge point: nl and ml determined in make_lgacco
c     we need the layer of the discharge kl
	kl = -1
	do k = 1,nlay
		if (casedata%ZL.le.real(k)*deltaz(nl,ml)) then
			kl = k
			exit
		endif
	enddo
	if (kl.le.0) kl = nlay
      
c     find current speed in discharge point (real location)
      udis = sqrt( ((uc(nl,ml,kl)+uc(nl,ml-1,kl))/2.)**2
     j           + ((vc(nl,ml,kl)+vc(nl-1,ml,kl))/2.)**2 )
      
c	correct segdis dependent of specified layer
      if (casedata%LayL(1:7).eq.'SURFACE') then
c         force surface discharge (buoyant plume)
          kl = 1
      elseif (casedata%LayL(1:6).eq.'BOTTOM') then
c         force bottom discharge (sinking plume)
          kl = nlay
      else
c         discharge will remain at the defined vertical layer
      endif
      segdis = segdis +(kl-1)*nosegl

      ntime = 2
	itime(1) = 0
	itime(2) = 86400
	call wridim 
     j (noseg,noq1,noq2,noq3,nosegl,nboun,segdis,udis,hydname)
	call wripoi (ipoint,4,noq,hydname,'.poi')

	call wrilen (itime(1),length,2,noq,hydname,'.len')
	call wrisur (nmax,mmax,nosegl,itime(1),surf,noseg,hydname,'.srf')
	call wribin (ntime,itime,volume,noseg,hydname,'.vol')
	call wribin (ntime,itime,area,noq,hydname,'.are')
	call wribin (ntime,itime,flow,noq,hydname,'.flo')

      write(*,*) 'Normal end'
      stop (0)
  900 write(*,*) 'Error: input file not found'
      stop (1)
  960 write(*,*) 'Allocation error'
      stop (2)
      end

